@@ -13,30 +13,26 @@ use std::rc::Rc;
 use chrono::Local;
 use std::time::{SystemTime, UNIX_EPOCH};
 use percent_encoding::{utf8_percent_encode, NON_ALPHANUMERIC};
-use std::thread;
+use mpris_player::PlaybackStatus;
 
-use crate::gui::microphone_thread::microphone_thread;
-use crate::gui::processing_thread::processing_thread;
-use crate::gui::http_thread::http_thread;
-use crate::gui::csv_song_history::{SongHistoryInterface, SongHistoryRecord};
-use crate::gui::thread_messages::{*, GUIMessage::*};
+use crate::core::microphone_thread::microphone_thread;
+use crate::core::processing_thread::processing_thread;
+use crate::core::http_thread::http_thread;
+use crate::core::thread_messages::{*, GUIMessage::*};
 
+use crate::utils::thread::spawn_big_thread;
 use crate::utils::pulseaudio_loopback::PulseaudioLoopback;
+use crate::utils::mpris_player::{get_player, update_song};
+
+use crate::gui::song_history_interface::SongHistoryInterface;
+use crate::utils::csv_song_history::SongHistoryRecord;
 
 #[cfg(windows)]
 use std::os::windows::process::CommandExt;
 
 use crate::fingerprinting::signature_format::DecodedSignature;
 
-fn spawn_big_thread<F, T>(argument: F) -> ()
-    where
-        F: std::ops::FnOnce() -> T,
-        F: std::marker::Send + 'static,
-        T: std::marker::Send + 'static {
-    thread::Builder::new().stack_size(32 * 1024 * 1024).spawn(argument).unwrap();
-}
-
-pub fn gui_main(recording: bool, input_file: Option<&str>) -> Result<(), Box<dyn Error>> {
+pub fn gui_main(recording: bool, input_file: Option<&str>, enable_mpris: bool) -> Result<(), Box<dyn Error>> {
     
     let application = gtk::Application::new(Some("com.github.marinm.songrec"),
         gio::ApplicationFlags::HANDLES_OPEN)
@@ -84,7 +80,7 @@ pub fn gui_main(recording: bool, input_file: Option<&str>) -> Result<(), Box<dyn
         spawn_big_thread(clone!(@strong gui_tx => move || { // http_rx
             http_thread(http_rx, gui_tx, microphone_tx_3);
         }));
-        
+
         // We create a callback for handling files to recognize opened
         // from the command line or through "xdg-open".
         
@@ -262,6 +258,11 @@ pub fn gui_main(recording: bool, input_file: Option<&str>) -> Result<(), Box<dyn
         
         let wipe_history_button: gtk::Button = builder.get_object("wipe_history_button").unwrap();
         let export_csv_button: gtk::Button = builder.get_object("export_csv_button").unwrap();
+
+        let mpris_player = if enable_mpris { get_player() } else { None };
+        if enable_mpris && mpris_player.is_none() {
+            println!("Unable to enable MPRIS support")
+        }
         
         // Thread-local variables to be passed across callbacks.
         
@@ -450,7 +451,7 @@ pub fn gui_main(recording: bool, input_file: Option<&str>) -> Result<(), Box<dyn
                 },
                 _ =>  { }
             }
-            
+
             match gui_message {
                 ErrorMessage(string) => {
                     if !(string == gettext("No match for this song") && microphone_stop_button.is_visible()) {
@@ -467,6 +468,9 @@ pub fn gui_main(recording: bool, input_file: Option<&str>) -> Result<(), Box<dyn
                     else {
                         network_unreachable.show_all();
                     }
+                    let mpris_status = if network_is_reachable { PlaybackStatus::Playing } else { PlaybackStatus::Paused };
+
+                    mpris_player.as_ref().map(|p| p.set_playback_status(mpris_status));
                 }
                 DevicesList(device_names) => {
                     let mut old_device_index = 0;
@@ -530,6 +534,8 @@ pub fn gui_main(recording: bool, input_file: Option<&str>) -> Result<(), Box<dyn
                     let song_name = Some(format!("{} - {}", message.artist_name, message.song_name));
         
                     if *youtube_query_borrow != song_name { // If this is already the last recognized song, don't update the display (if for example we recognized a lure we played, it would update the proposed lure to a lesser quality)
+
+                        mpris_player.as_ref().map(|p| update_song(p, &message));
                         
                         if microphone_stop_button.is_visible() {
 
@@ -543,7 +549,10 @@ pub fn gui_main(recording: bool, input_file: Option<&str>) -> Result<(), Box<dyn
                         song_history_interface.add_column_and_save(SongHistoryRecord {
                             song_name: song_name.as_ref().unwrap().to_string(),
                             album: message.album_name.as_ref().unwrap_or(&"".to_string()).to_string(),
-                            recognition_date: Local::now().format("%c").to_string()
+                            recognition_date: Local::now().format("%c").to_string(),
+                            track_key: message.track_key,
+                            release_year: message.release_year.as_ref().unwrap_or(&"".to_string()).to_string(),
+                            genre: message.genre.as_ref().unwrap_or(&"".to_string()).to_string(),
                         });
 
                         recognized_song_name.set_markup(&format!("<b>{}</b>", glib::markup_escape_text(song_name.as_ref().unwrap())));
